@@ -27,7 +27,7 @@ __author__ = ["Rafael Pastrana"]
 __copyright__ = "Copyright 2020 - Princeton University"
 __license__ = "MIT License"
 __email__ = "arpj@princeton.edu"
-__version__ = "0.6.0"
+__version__ = "0.7.0"
 
 
 # Directories
@@ -46,10 +46,11 @@ COLORS = {"tension": (227, 6, 75),  # red
           "node_black": (0, 0, 0),  # black
           "node_support": (0, 150, 10),  # green
           "node_origin": (155, 100, 255),  # purple
-          "edge_trail": (127, 0, 127),  # purple
-          "edge_deviation": (127, 255, 0),  # lime
-          "support_force": (75, 75, 75),  # dark gray / green (0, 150, 10)
-          "load": (0, 150, 10),  # green / light green (50, 250, 100)
+          "edge_trail": (180, 0, 255),  # purple
+          "edge_deviation": (127, 255, 0),  # lime  (127, 155, 0)
+          "edge_deviation_indirect": (255, 100, 47),  # coral
+          "support_force": (75, 75, 75),  # dark gray
+          "load": (0, 150, 10),  # green
           "trail": (250, 80, 210),  # pink
           "auxiliary_trail": (255, 155, 15)}  # orange
 
