@@ -1,3 +1,6 @@
+from math import fabs
+from math import copysign
+
 from compas_cem.elements import Edge
 
 
@@ -12,7 +15,13 @@ class DeviationEdge(Edge):
     def __repr__(self):
         """
         """
-        return "{0}(force={1!r})".format(self.__class__.__name__, self.attributes["force"])
+        force = self.attributes["force"]
+        msg = "{name}(force={force!r}, state={state!r})"
+        info = {"name": self.__class__.__name__,
+                "force": fabs(force),
+                "state": int(copysign(1, force))}
+
+        return msg.format(**info)
 
 # ==============================================================================
 # Main
