@@ -1,9 +1,7 @@
 import numpy as np
 
-from autograd import grad as agrad
 
-
-__all__ = ["grad_finite_differences_numpy",
+__all__ = ["grad_finite_differences",
            "grad_autograd"]
 
 # ------------------------------------------------------------------------------
@@ -11,11 +9,11 @@ __all__ = ["grad_finite_differences_numpy",
 # ------------------------------------------------------------------------------
 
 
-def grad_autograd(x, grad, grad_func):
+def grad_autograd(x, grad, grad_func, **kwargs):
     """
-    Calculates the gradient with automatic differentiation. And updated grad in-place.
+    Calculate the gradient with automatic differentiation.
+    This function updates grad in place.
     """
-    grad_func = agrad(grad_func)
     grad[:] = grad_func(x)
 
     return grad
@@ -25,22 +23,28 @@ def grad_autograd(x, grad, grad_func):
 # ------------------------------------------------------------------------------
 
 
-def grad_finite_differences_numpy(x, grad, x_func, step_size):
+def grad_finite_differences(x, grad, x_func, step_size, **kwargs):
     """
-    Approximates the gradient of a blackbox function using finite differences.
+    Approximate the gradient of a blackbox function using forward finite differences.
+    This function updates grad in place.
     """
-    # finite difference
     fx0 = x_func(x)
+    # NOTE: We make an editable copy of x because NLOpt makes x a read-only vector
+    _x = np.copy(x)
 
-    for i in range(len(x)):  # vectorize with numpy?
+    for i in range(len(x)):
 
-        _x = np.copy(x)
+        _xi = _x[i]
         _x[i] += step_size
 
-        fx1 = x_func(_x)  # bottleneck
+        fx1 = x_func(_x)
 
         delta_fx = (fx1 - fx0) / step_size
         grad[i] = delta_fx
+        _x[i] = _xi
+
+    return grad
+
 
 # ------------------------------------------------------------------------------
 # Main
